package com.ahazou.photo_editor_sdk

import android.app.Activity
import android.content.Intent
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import ly.img.android.pesdk.PhotoEditorSettingsList
import ly.img.android.pesdk.assets.font.basic.FontPackBasic
import android.net.Uri
import androidx.annotation.AnyRes
import com.ahazou.photo_editor_sdk.instagram_ui.InstagramUIBuilder
import com.ahazou.photo_editor_sdk.utils.CustomImageStickerAsset
import com.ahazou.photo_editor_sdk.utils.ExampleConfigUtility.createInitialPesdkSettingsList
import com.ahazou.photo_editor_sdk.utils.Utils
import com.bumptech.glide.Glide
import com.bumptech.glide.load.DataSource
import com.bumptech.glide.load.engine.GlideException
import com.bumptech.glide.request.RequestListener
import com.bumptech.glide.request.target.Target
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.PluginRegistry
import ly.img.android.pesdk.backend.decoder.Decoder
import ly.img.android.pesdk.backend.decoder.ImageSource
import ly.img.android.pesdk.backend.model.config.ImageStickerAsset
import ly.img.android.pesdk.backend.model.constant.Directory.DCIM
import ly.img.android.pesdk.backend.model.state.AssetConfig
import ly.img.android.pesdk.backend.model.state.LoadSettings
import ly.img.android.pesdk.backend.model.state.SaveSettings
import ly.img.android.pesdk.ui.activity.ImgLyIntent
import ly.img.android.pesdk.ui.model.state.*
import ly.img.android.pesdk.ui.panels.item.ImageStickerItem
import ly.img.android.pesdk.ui.panels.item.StickerCategoryItem
import ly.img.android.pesdk.ui.panels.item.ToolItem
import java.io.File
import java.lang.ref.WeakReference
import java.util.*


/** PhotoEditorSdkPlugin */
class PhotoEditorSdkPlugin:
        ActivityAware,
        FlutterPlugin,
        MethodCallHandler,
        PluginRegistry.ActivityResultListener {

  companion object {
    lateinit var weakActivity: WeakReference<Activity>
    lateinit var arrLogos: ArrayList<CustomImageStickerAsset>
    lateinit var arrStickers: ArrayList<CustomImageStickerAsset>
    lateinit var resultClass: Result
  }

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    val channel = MethodChannel(flutterPluginBinding.binaryMessenger, "photo_editor_sdk")
    channel.setMethodCallHandler(PhotoEditorSdkPlugin())
  }

  override fun onDetachedFromActivity() {
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    weakActivity = WeakReference(binding.activity)

  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
  }

  override fun onDetachedFromActivityForConfigChanges() {
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    resultClass = result
    when ( call.method ) {
      "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
      "addAllContents" -> {
        val args = call.arguments as Map<*, *>
        val logos: ArrayList<*> = args["logos"] as ArrayList<*>
        val stickers: ArrayList<*> = args["stickers"] as ArrayList<*>
        arrLogos = Utils.retrieveAssets(logos, weakActivity.get()!!)
        arrLogos.add(CustomImageStickerAsset(
                ImageStickerAsset("imgly_ic_add_new_logo", R.drawable.ic_add_user_logo),
                getResourceImage(R.drawable.ic_add_user_logo)))
        arrStickers = Utils.retrieveAssets(stickers, weakActivity.get()!!)
      }
      "editImageByUrl" -> {
//        val args = call.arguments as? String
//        val uri = Uri.parse(args)
        openEditor(/*uri*/)
      }
      "editImageByFile" -> {
//        val args = call.arguments as? String
//        val file = File(args)
//        val uri = Uri.fromFile(file)
        openEditor(/*uri*/)
      }
      else -> result.notImplemented()
    }
  }

//  private fun createPesdkSettingsList(inputImage: Uri?) =
//          PhotoEditorSettingsList()
//                  .configure<UiConfigText> {
//                    it.setFontList(FontPackBasic.getFontPack())
//                  }
//                  .configure<SaveSettings> {
//                    it.setExportDir(DCIM, "AhazouPro")
//                    it.setExportPrefix("result_")
//                    it.savePolicy = SaveSettings.SavePolicy.RETURN_ALWAYS_ONLY_OUTPUT
//                  }
//                  .configure<LoadSettings> {
//                    it.source = inputImage
//                    it.isDeleteProtectedSource = true
//                  }
//                  .configure<UiConfigTheme> {
//                    it.theme =  R.style.Imgly_Theme_TopActionBar_NoFullscreen
//                  }
//                  .configure<UiConfigMainMenu> {
//                    it.setToolList(
//                            ToolItem("imgly_tool_sticker_selection",
//                                    R.string.pesdk_sticker_title_name, ImageSource.create(R.drawable.imgly_icon_tool_sticker)),
//                            ToolItem("imgly_tool_text",
//                                    R.string.pesdk_text_title_name, ImageSource.create(R.drawable.imgly_icon_tool_text))
//                    )
//                  }
//                  .configure<UiConfigSticker> {
//                    val arrLogoSticker = ArrayList<StickerCategoryItem>()
//                    if( arrLogos.isNotEmpty() ) {
//                      arrLogoSticker.add(StickerCategoryItem(
//                              "logos_id",
//                              "logos",
//                              ImageSource.create(arrLogos[0].uri),
//                              arrLogos.map { s -> ImageStickerItem.createFromAsset(s.image) }
//                      ))
//                    }
//
//                    if( arrStickers.isNotEmpty() ) {
//                      arrLogoSticker.add(StickerCategoryItem(
//                              "stickers_id",
//                              "stickers",
//                              ImageSource.create(arrStickers[0].uri),
//                              arrStickers.map { s -> ImageStickerItem.createFromAsset(s.image) }
//                      ))
//                    }
//                    it.setStickerLists(arrLogoSticker)
//                  }
//                  .configure<AssetConfig> {
//                    arrLogos.map { logo -> it.addAsset(logo.image) }
//                    arrStickers.map { sticker -> it.addAsset(sticker.image) }
//                  }

  val PESDK_RESULT = 1000

  private fun openEditor(/*inputImage: Uri?*/) {
    val settingsList = createInitialPesdkSettingsList()

    settingsList
        .getSettingsModel(LoadSettings::class.java)
        .setSource(getResourceImage(R.drawable.sample_image), true)

    InstagramUIBuilder(weakActivity.get())
            .setSettingsList(settingsList)
            .startActivityForResult(weakActivity.get(), PESDK_RESULT)
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent) : Boolean {
    if (resultCode == Activity.RESULT_OK && requestCode == PESDK_RESULT) {
      data.getParcelableExtra<Uri?>(ImgLyIntent.RESULT_IMAGE_URI)?.also {
        weakActivity.get()!!.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE).setData(it))
        val filePath = it.path
        val file = File(filePath)
        val b = file.readBytes()
        resultClass.success(b)
      }
    }
    return true
  }

  private fun getResourceImage(@AnyRes resourceId: Int) : Uri
          = Decoder.resourceToUri(weakActivity.get()!!.resources, resourceId)

}